package jp.co.sds_corp.util.window;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

/**
* ステータスバーです。
* @author	小見山　聡
*/
public class StatusBar extends JPanel
{
	int		nClm;
	int		nHeight = 24;
	Vector<Integer>	vecWidth = new Vector<Integer>();
	Vector<JComponent>	vecLbl = new Vector<JComponent>();
	public enum Status {
		OK,
		NG,
		UNKNOWN
	}
	public enum Type {
		LABEL,
		PROGRESS;
	}

	/**
	* ステータスバーを初期化します。
	*/
	public StatusBar() {
		this( 1 );
	}

	/**
	* ステータスバーを指定されたカラム数で初期化します。
	* @param	nClm	カラム数
	*/
	public StatusBar( int nClm ) {
		super();
		this.nClm = nClm;
		setLayout( null );
		setPreferredSize( new Dimension( 10000, 24 ));
		for ( int i = 0; i < nClm; i++ ) {
			vecWidth.add( Integer.valueOf( -1 ));
			JLabel lbl = new JLabel( " " );
			lbl.setBorder( BorderFactory.createLoweredBevelBorder() );
			add( lbl );
			vecLbl.add( lbl );
		}
	}

	int		nX = -1, nY = -1, nW = -1, nH = -1;
	public void setBounds( int x, int y, int w, int h ) {
		if ( x == nX && y == nY && w == nW && h == nH ) {
			return;
		}
		super.setBounds( x, y, w, h );

		int[]	tmpW = new int[nClm];
		int		nAll = 0;
		int		nNul = 0;
		JComponent c0 = (JComponent )vecLbl.get( 0 );
		Insets insets = c0.getBorder().getBorderInsets( c0 );

		int nWidth;
		for ( int i = 0; i < nClm; i++ ) {
			Integer iw = (Integer )vecWidth.get( i );
			nWidth = iw.intValue();
			if ( nWidth >= 0 ) {
				tmpW[i] = nWidth;
				nAll += nWidth;
			} else {
				nNul++;
			}
		}
		if ( nAll < w && nNul > 0 ) {
			for ( int i = 0; i < nClm; i++ ) {
				nWidth = ((Integer )vecWidth.get( i )).intValue();
				if ( nWidth < 0 ) {
					tmpW[i] = ( w - nAll ) / nNul;
				}
			}
		}

		int		nX = 0;
		for ( int i = 0; i < nClm; i++ ) {
			JComponent c = (JComponent )vecLbl.get( i );
			c.setBounds( nX, 0, tmpW[i], h );
			nX += tmpW[i];
		}
		nX = x;
		nY = y;
		nW = w;
		nH = h;
	}

	public int getHeight() {
		return nHeight;
	}
	public void setHeight( int nHeight ) {
		this.nHeight = nHeight;
		setPreferredSize( new Dimension( 10000, nHeight ));
	}

	public int getWidth( int nPos ) {
		Integer i = (Integer )vecWidth.get( nPos );
		return i.intValue();
	}
	public void setWidth( int nPos, int nWidth ) {
		if ( nPos >= nClm ) {
			throw( new ArrayIndexOutOfBoundsException( "pos must be < " + nClm ));
		}
		vecWidth.set( nPos, Integer.valueOf( nWidth ));
		setBounds( getBounds() );
	}

	public int getHorizontalAlignment( int nPos ) {
		JComponent c = (JComponent )vecLbl.get( nPos );
		if ( !( c instanceof JLabel )) {
			return -1;
		}
		JLabel lbl = (JLabel )c;
		return lbl.getHorizontalAlignment();
	}
	public void setHorizontalAlignment( int nPos, int nAlign ) {
		JComponent c = (JComponent )vecLbl.get( nPos );
		if ( !( c instanceof JLabel )) {
			return;
		}
		JLabel lbl = (JLabel )c;
		lbl.setHorizontalAlignment( nAlign );
	}

	public String getText( int nPos ) {
		JComponent c = (JComponent )vecLbl.get( nPos );
		if ( !( c instanceof JLabel )) {
			return null;
		}
		JLabel lbl = (JLabel )c;
		return lbl.getText();
	}
	public void setText( int nPos, String text ) {
		JComponent c = (JComponent )vecLbl.get( nPos );
		if ( !( c instanceof JLabel )) {
			return;
		}
		JLabel lbl = (JLabel )c;
		lbl.setText( text );
	}

	public void setText( int nPos, String text, Status nStat ) {
		setText( nPos, text );
		setStatus( nPos, nStat );
	}

	public void setStatus( int nPos, Status nStat ) {
		JComponent c = (JComponent )vecLbl.get( nPos );
		if ( !( c instanceof JLabel )) {
			return;
		}
		JLabel lbl = (JLabel )c;
		String icon = "";
		switch ( nStat ) {
		case OK: icon = "/resource/stat_ok_t.gif"; break;
		case NG: icon = "/resource/stat_ng_t.gif"; break;
		case UNKNOWN: icon = "/resource/stat_uk_t.gif"; break;
		}
		URL img = getClass().getResource( icon );
		Insets ins = lbl.getBorder().getBorderInsets( lbl );
		lbl.setIcon( new SizedIcon( img, nHeight - ins.top - ins.bottom ));
	}

	public void setType( int nPos, Type nType ) {
		JComponent c = null;
		switch ( nType ) {
		case LABEL: c = new JLabel(); break;
		case PROGRESS: c = new JProgressBar(); break;
		}
		if ( c != null ) {
			setObject( nPos, c );
		}
	}

	public void setObject( int nPos, JComponent c ) {
		JComponent co = (JComponent )vecLbl.get( nPos );
		remove( co );
		c.setBorder( BorderFactory.createLoweredBevelBorder() );
		add( c );
		vecLbl.set( nPos, c );
		setBounds( getBounds() );
	}

	public JComponent getObject( int nPos ) {
		JComponent c = (JComponent )vecLbl.get( nPos );
		return c;
	}

	public int getColumn() {
		return nClm;
	}

	public void setColumn( int nClm ) {
		for ( int i = 0; i < this.nClm; i++ ) {
			JComponent c = (JComponent )vecLbl.get( i );
			remove( c );
		}
		for ( int i = 0; i < nClm; i++ ) {
			JComponent c = null;
			if (( i + 1 ) > vecWidth.size() ) {
				vecWidth.add( Integer.valueOf( -1 ));
				c = new JLabel( " " );
				c.setBorder( BorderFactory.createLoweredBevelBorder() );
				vecLbl.add( c );
			} else {
				c = (JComponent )vecLbl.get( i );
			}
			add( c );
		}
		vecWidth.setSize( nClm );
		vecLbl.setSize( nClm );
		this.nClm = nClm;
		setBounds( getBounds() );
	}

	static int nCnt = 1;
	static StatusBar stat;
	static JProgressBar prg;
	public static void main( String[] args ) {
		JFrame frame = new JFrame();
		frame.setBounds( 0, 0, 500, 100 );
		Container pane = frame.getContentPane();
		pane.setLayout( new BorderLayout() );
		JButton btn = new JButton( "test" );
		btn.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				nCnt++;
				if ( nCnt > 5 ) {
					nCnt = 1;
				}
				stat.setColumn( nCnt );
				if ( nCnt == 2 ) {
					stat.setWidth( 1, 50 );
					stat.setText( 1, "Test" );
				}
				if ( nCnt == 3 ) {
					stat.setStatus( 2, Status.UNKNOWN );
				}
				if ( nCnt == 4 ) {
					stat.setType( 3, Type.PROGRESS );
					JProgressBar prg = (JProgressBar )stat.getObject( 3 );
					prg.setMinimum( 0 );
					prg.setMaximum( 100 );
					prg.setValue( 30 );
					prg.setString( "30%" );
					prg.setStringPainted( true );
				}
			}
		} );
		pane.add( btn, BorderLayout.NORTH );
		stat = new StatusBar();
		pane.add( stat, BorderLayout.SOUTH );
		stat.setText( 0, "Ready", Status.OK );
		frame.setVisible( true );
	}

}

